const fs = require("fs");
const child_process = require("child_process");
const path = require("path");
const process = require("process");
const utils = require("./utils.js");
const Constants = require("./constants.js");

module.exports.register = function(program) {
	program
		.command("build")
		.description("Build AruJs Application, Located Inside the <directory> Directory.")
		.option("--directory <directory>", "Directory Containing Your Project (default: Current Directory)")
		.option("--outdir <directory>", "Directory To Put The Builds In (default './build')")
		.action(async function(command) {
			if (!command?.directory) command.directory = process.cwd();
			if (!command?.outdir) command.outdir = path.join(process.cwd(), "build");
			try {
				utils.ValidateProject(command.directory);

				var AruJsConf = utils.json.readJson(path.resolve(path.join(command.directory, Constants.configFile)));
				if (!AruJsConf?.arujs?.binPath) {
					program.error(`Cannot Find The "binPath" property in AruJs Config!`, { exitCode: 1 });
					return;
				}

				let currDir = process.cwd();
				process.chdir(command.directory);
				var binPath = path.resolve(AruJsConf.arujs.binPath);
				try {
					fs.mkdirSync(binPath, { recursive: true });
				} catch(err) {
					console.log(err);
				}
				process.chdir(currDir);

				let dirName = path.basename(command.directory);
				for (os in Constants.binary) {
					for (arch in Constants.binary[os]) {
						let filePath = path.join(binPath, Constants.binary[os][arch]);
						if (!fs.existsSync(filePath)) continue;

						let _outdir = path.join(command.outdir, `${dirName}-${os}-${arch}/${dirName}.aruapp`);
						console.log(_outdir);
						fs.mkdirSync(_outdir, { recursive: true });
						utils.copyRecursiveSync(command.directory, _outdir, [ binPath ]);
						fs.copyFileSync(filePath, path.join(command.outdir, `${dirName}-${os}-${arch}/${dirName}${os == "windows" ? ".exe" : ""}`));
					}
				}
			} catch (err) {
				program.error(`Error: ${err}`, { exitCode: 1 });
				return;
			}
		});
}

