const fs = require("fs");
const child_process = require("child_process");
const path = require("path");
const process = require("process");
const utils = require("./utils.js");
const Constants = require("./constants.js");

module.exports.register = function(program) {
	program
		.command("run")
		.description("Run AruJs Application, Located Inside <directory>/Current Directory.")
		.option("--directory <directory>", "Directory Containing Your Project (default: Current Directory)")
		.option("--is-debug", "Enables Debugging Specific Options, Like Element Inspector")
		.option("--enable-hotreloading", "Reloads The Opened Page On Change In <directory>")
		.action(async function(command) {
			if (!command?.isDebug) command.isDebug = false;
			if (!command?.directory) command.directory = process.cwd();
			if (!command?.enableHotreloading) command.enableHotreloading = false;
			try {
				command.directory = path.resolve(command.directory);
				utils.ValidateProject(command.directory);

				if (Constants.arch == "")	{
					program.error(`AruJs isn't Available For "${process.arch}" Arch.`, { exitCode: 1 });
					return;
				} else if (Constants.os == "") {
					program.error(`AruJs isn't Available For "${process.platform}" OS.`, { exitCode: 1 });
					return;
				}

				let stats = fs.lstatSync(path.resolve(command.directory));
				if (!stats.isDirectory()) {
					program.error(`"${command.directory}" is Not a Directory!`, { exitCode: 1 });
				} else {
					const AruJsConf = utils.json.readJson(path.join(command.directory, Constants.configFile));
					if (!AruJsConf?.arujs?.binPath) {
						program.error(`Cannot Find The "binPath" property in AruJs Config!`, { exitCode: 1 });
						return;
					}

					console.log(`Running AruJs Application inside "${command.directory}" Directory With Options:
  Platform: ${Constants.os}
  Architecture: ${Constants.arch}
  Debug Mode: ${command.isDebug}
  Hot-Reloading: ${command.enableHotreloading}\n`);

					let currDir = process.cwd();
					process.chdir(command.directory);
					var binPath = path.resolve(AruJsConf.arujs.binPath);
					try {
						fs.mkdirSync(binPath, { recursive: true });
					} catch(err) {
						console.log(err);
					}
					process.chdir(currDir);

					try {
						child_process.execFileSync(
							path.join(binPath, Constants.binary[Constants.os][Constants.arch]),
							[
								command.directory,
								command.isDebug ? "--is-debug" : "",
								command.enableHotreloading ? "--enable-hotreloading" : ""
							],
							{ cwd: path.resolve("./"), shell: false }
						);
					} catch(err) {
						console.log(err);
						program.error(`STDOUT: ${err?.stdout?.toString()}\nSTDERR: ${err?.stderr?.toString()}`, { exitCode: 1 });
						return;
					}
				}
			} catch (err) {
				program.error(`Error: ${err}`, { exitCode: 1 });
				return;
			}
		});
}
