const fs = require("fs");
const path = require("path");
const process = require("process");
const utils = require("./utils.js");
const Constants = require("./constants.js");

module.exports.register = function(program) {
	program
		.command("use")
		.description("Use A Specific Version Of AruJs Core, The Version Is In Format Of Major.Minor.Patch-Stability.")
		.option("--directory <directory>", "Directory Containing Your Project (default: Current Directory)")
		.option("--version <version>", "Version To Use, Like: 0.0.2-dev or 0.1.4-stable")
		.action(async function(command) {
			if (!command?.directory) command.directory = process.cwd();
			try {
				utils.ValidateProject(command.directory);

				var AruJsConf = utils.json.readJson(path.resolve(path.join(command.directory, Constants.configFile)));
				if (!AruJsConf?.arujs?.binPath) {
					program.error(`Cannot Find The "binPath" property in AruJs Config!`, { exitCode: 1 });
					return;
				}

				if (Constants.arch == "")	{
					program.error(`AruJs isn't Available For "${process.arch}" Arch.`, { exitCode: 1 });
					return;
				} else if (Constants.os == "") {
					program.error(`AruJs isn't Available For "${process.platform}" OS.`, { exitCode: 1 });
					return;
				} else if (!command?.version || !AruJsConf?.arujs?.version) {
					command.version = Constants.latestStable;
					console.log(`No Version Specified, Using "${command.version}" instead.`);
				}

				let currDir = process.cwd();
				process.chdir(command.directory);
				var binPath = path.resolve(AruJsConf.arujs.binPath);
				try {
					fs.mkdirSync(binPath, { recursive: true });
				} catch(err) {
					console.log(err);
				}
				process.chdir(currDir);

				let hasDownloadedAtleastOnce = false;
				for (os in Constants.binary) {
					for (arch in Constants.binary[os]) {
						let filePath = path.join(binPath, Constants.binary[os][arch]);
						console.log(`Downloading Binary For OS: ${os}, Arch: ${arch}, File: ${path.basename(filePath)}`);
						let url = Constants.url.replace(/\$TAG_NAME/g, "v" + command.version).replace(/\$FILE_NAME/g, path.basename(filePath));
						await utils.downloadFileTo(url, filePath).then(function() {
							console.log("Done!");
							hasDownloadedAtleastOnce = true;
						}).catch(function(statusCode) {
							console.error(`Download Failed With Status Code: ${statusCode}`);
						});
					}
				}

				if (hasDownloadedAtleastOnce) {
					AruJsConf.arujs.version = command.version;
					utils.json.writeJson(path.resolve(path.join(command.directory, Constants.configFile)), AruJsConf);
				}
			} catch (err) {
				program.error(`Error: ${err}`, { exitCode: 1 });
				return;
			}
		});
}
