const fs = require("fs");
const path = require("path");
const process = require("process");
const constants = require("./constants.js");
const { https } = require('follow-redirects');
const CONFIG_FILE = constants.configFile;

module.exports.IsAruJsProject = function(directory) {
	try {
		return fs.existsSync(path.join(directory, CONFIG_FILE));
	} catch(err) {
		console.error(`Error Validating Project: ${err}`);
		return false;
	}
}

module.exports.ValidateProject = function(directory) {
	if (!module.exports.IsAruJsProject(directory)) {
        console.error(`Unable to find ${CONFIG_FILE}. Please check whether the directory "${directory}" has a AruJs project.`);
        process.exit(1);
	}
}

module.exports.json = {
	readJson: function(filePath) {
		try {
			if (!fs.existsSync(filePath)) {
				console.log(`File: ${filePath} doesn't exist!`);
				return {};
			}

			let jsonText = fs.readFileSync(filePath, { encoding: 'utf8' });
			return JSON.parse(jsonText);
		} catch(err) {
			console.log(`Unable To Read Json: ${err}`);
			return {};
		}
	},
	writeJson: function(filePath, jsonObj) {
		try {
			let jsonText = JSON.stringify(jsonObj, null, '\t');
			fs.writeFileSync(filePath, jsonText, { encoding: 'utf8' });
		} catch(err) {
			console.log(err);
		}
	}
}

/**
 * Look ma, it's cp -R.
 * @param {string} src  The path to the thing to copy.
 * @param {string} dest The path to the new copy.
 */
module.exports.copyRecursiveSync = function(src, dest, ignore=[]) {
	var exists = fs.existsSync(src);
	var stats = exists && fs.statSync(src);
	var isDirectory = exists && stats.isDirectory();
	if (isDirectory) {
		try { fs.mkdirSync(dest); }
		catch(err) {}
		fs.readdirSync(src).forEach(function(childItemName) {
			for (var i = 0; i < ignore.length; i++) {
				if (path.resolve(ignore[i]) == path.resolve(path.join(src, childItemName))) return;
			}
			module.exports.copyRecursiveSync(path.join(src, childItemName), path.join(dest, childItemName), ignore);
		});
	} else {
		fs.copyFileSync(src, dest);
	}
};

module.exports.downloadFileTo = function(url, filePath) {
	return new Promise((resolve, reject) => {
		const file = fs.createWriteStream(filePath);
		https.get(url, function (response) {
			const { statusCode } = response;
			if (statusCode != 200) {
				file.end();
				try { fs.unlinkSync(filePath); }
				catch(err) {}
				reject(statusCode);
			}
			response.pipe(file);
			response.on('end', () => {
				file.end();
				fs.chmodSync(filePath, 0777);
				resolve();
			});
		}).on('error', (e) => {
			console.error(e);
		});
	});
}

